/* This piece of code is called 3 times by analysis.do (once for concrete 
   producers, once for cardboard box producers, and once for the pooled sample).
   The code does 3 main things.  
   
   First, it runs regressions (and stores coefficients 
   and standard errors) which are of the form: 
 
   tfpq of downtream plant = fixed effects
                            + average tfpq of upstream plants of the d.stream plant 
                            + average materials price paid by other other plants
                            + within-supplier price deviation
   OR 
   input price of the d.stream plant
                           = fixed effects
                            +  average tfpq of upstream plants of the d.stream plant 
                            + average materials price paid by other other plants
                            + within-supplier price deviation
   OR 
   phi of the d.stream plant
                           = fixed effects
                            +  average tfpq of upstream plants of the d.stream plant 
                            + average materials price paid by other other plants
                            + within-supplier price deviation


   Second, it computes the fraction of dispersion in Phi, TFPQ or input prices that can be 
   explained by a) the geographical location (Census division) of the plant b) the 
   average productivity of the upstream plants and c) "price discriminatory" behavior 
   of the upstream plants.  
 
   Finally,the do-file tries to answer whether the decline in dispersion is solely due 
   to the reduction in degrees of freedom (it will be clearer, below, what I mean by this)*/



foreach w in W UW { 
  foreach x in price_coef_ tfp_coef_ phi_coef_ price_se_ tfp_se_ phi_se_  price_r2a_ tfp_r2a_ phi_r2a_ divOnly_r2a_ divOnly_r2a_P_  divOnly_r2a_Phi_ {
    gen `x'`w'=. 
  }
}

gen price_coef_W_MK=. /* MK denotes that the average within-supplier deviation is included in the regression */
gen tfp_coef_W_MK=.
gen phi_coef_W_MK=.
gen price_coef_UW_MK=.
gen tfp_coef_UW_MK=.
gen phi_coef_UW_MK=.

gen price_se_W_MK=. 
gen tfp_se_W_MK=.
gen phi_se_W_MK=.
gen price_se_UW_MK=.
gen tfp_se_UW_MK=.
gen phi_se_UW_MK=.

gen price_coef_W_PP=. /* PP denotes that the average local materials price is included in the regression */
gen tfp_coef_W_PP=.
gen phi_coef_W_PP=.
gen price_coef_UW_PP=.
gen tfp_coef_UW_PP=.
gen phi_coef_UW_PP=.

gen price_se_W_PP=.
gen tfp_se_W_PP=.
gen phi_se_W_PP=.
gen price_se_UW_PP=.
gen tfp_se_UW_PP=.
gen phi_se_UW_PP=.

gen samplesize=.

forvalues x=1/4 {
  foreach y in disp_Price disp_TFP disp_Phi  disp_FAKE_Price  disp_FAKE_TFP disp_FAKE_Phi mn_disp_FAKE_Price  mn_disp_FAKE_TFP mn_disp_FAKE_Phi sd_disp_FAKE_Price sd_disp_FAKE_TFP sd_disp_FAKE_Phi  {
    foreach w in W UW {
     gen `y'_`w'_`x'=.
  } 
 }
}

gen kkk=.
gen ppp=.
gen iii=.
gen jjj=.
gen xxx=.
gen ddd=0
gen www=.

local mkpvar= " " 
cap gen ones=1
gen weightvar=.

/* This section of the code performs the first and second steps (as outlined above) */

/* vv is the variable used on the left-hand side of the regression.  
   Of the tables that appear in the paper, only the regressions with
   vv=priceI_CFS are used. 
   */

foreach vv in priceI_CFS ltfpqd ltfpq3 {

if "`vv'"=="ltfpqd" {
  local name1="tfp"
  local name2="_"
  local name3="TFP"
}
if "`vv'"=="ltfpq3" {
  local name1="phi"
  local name2="_Phi_"
  local name3="Phi"
}

if "`vv'"=="priceI_CFS" {
  local name1="price"
  local name2="_P_"
  local name3="Price"
}

forvalues ww=0/1 {   /* do you weight observations by revenue of the downstream plant? */

if `ww'==0 {
  replace weightvar=ones
  local www="UW"
  local www2=""
}
if `ww'==1 {
  replace weightvar=tvsd
  local www="W"
  local www2="_W"
}
forvalues pp=0/2 {    /* the local macro variable pp controls whether we include the local price variable.  The default (pp=0) is to not to. */
forvalues kk=0/1 {   /* when k=0 do not include the "price discrimination term */
  local mkpvar = "ones"
  if `kk'==1 {
   local mkpvar = "avgMarkupMean" 
  } 
forvalues ii=1/2 {    /* the local macro variable ii controls whether we discard observations for which there are 
                         many candidate establishments in the downstream zip-code.  The default is not to discard any 
                        observations */
 forvalues jj=1/3 {   /* the local macro variable jj controls the cut-off for removing plants based on the share 
                         of materials purchased that are within the firm (default is jj=2, which corresponds to a
                        15%  cutoff)*/
 forvalues x=0/2 {    /* the local macro variable x controls whether we restrict the sample by matching on prices.
                       The default is to not restrict the sample. */
 local y=(`ii'-1)*18+(`jj'-1)*6+`x'*2+1+`kk'*54+`pp'*108
 local z=`y'-1
 local zz=`y'+1
 local ppvar = "ones"
  if `pp'==1 & `ww'==0 {
   local ppvar = "price50_`x'_`ii'_`jj'"
  } 
  if `pp'==2 & `ww'==0 {
   local ppvar = "price100_`x'_`ii'_`jj'"
  } 
  if `pp'==1 & `ww'==1 {
   local ppvar = "price50_`x'_`ii'_`jj'_W"
  } 
  if `pp'==2 & `ww'==1 {
   local ppvar = "price100_`x'_`ii'_`jj'_W"
  } 
qui { 
  qui reg `vv'_`x'_`ii'_`jj'`www2' `mkpvar' `ppvar' if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1  [aw=weightvar]
  qui replace disp_`name3'_`www'_1=r(sd) in `y'/`y'
  drop e_e
  qui reg `vv'_`x'_`ii'_`jj'`www2' tfpq_Up_W_`x'_`ii'_`jj'`www2' `mkpvar' `ppvar'  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  replace disp_`name3'_`www'_3=r(sd) in `y'/`y'
  drop e_e
  replace `name1'_coef_`www'=_b[tfpq_Up_W] in `y'/`y'
  replace `name1'_se_`www'=_se[tfpq_Up_W] in `y'/`y'
  if `kk'==1 {
   replace `name1'_coef_`www'_MK=_b[avgMarkupMean] in `y'/`y'
   replace `name1'_se_`www'_MK=_se[avgMarkupMean] in `y'/`y'
  }
  if `pp'==1 {
   replace `name1'_coef_`www'_PP=_b[price50_`x'_`ii'_`jj'] in `y'/`y'
   replace `name1'_se_`www'_PP=_se[price50_`x'_`ii'_`jj'] in `y'/`y'
  }
  if `pp'==2 {
   replace `name1'_coef_`www'_PP=_b[price100_`x'_`ii'_`jj'] in `y'/`y'
   replace `name1'_se_`www'_PP=_se[price100_`x'_`ii'_`jj'] in `y'/`y'
  }
  qui replace `name1'_r2a_`www'=e(r2_a) in `y'/`y'

  replace iii=`ii' in `y'/`y'
  replace jjj=`jj' in `y'/`y'
  replace xxx=`x' in `y'/`y'
  replace kkk=`kk' in `y'/`y'
  replace ppp=`pp' in `y'/`y'
  replace www=`ww' in `y'/`y'
  local y=`y'+1
  local z=`y'-1
  local zz=`y'+1

  reg `vv'_`x'_`ii'_`jj'`www2' tfpq_Up_W_`x'_`ii'_`jj'`www2'   i.divd  `mkpvar' `ppvar'  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  replace ddd=1 in `y'/`y'
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  replace disp_`name3'_`www'_4=r(sd) in `z'/`z'
  drop e_e
  replace `name1'_coef_`www'=_b[tfpq_Up_W] in `y'/`y'
  replace `name1'_se_`www'=_se[tfpq_Up_W] in `y'/`y'
  if `kk'==1 {
   replace `name1'_coef_`www'_MK=_b[avgMarkupMean] in `y'/`y'
   replace `name1'_se_`www'_MK=_se[avgMarkupMean] in `y'/`y'
  }
  if `pp'==1 {
   replace `name1'_coef_`www'_PP=_b[price50_`x'_`ii'_`jj'] in `y'/`y'
   replace `name1'_se_`www'_PP=_se[price50_`x'_`ii'_`jj'] in `y'/`y'
  }
  if `pp'==2 {
   replace `name1'_coef_`www'_PP=_b[price100_`x'_`ii'_`jj'] in `y'/`y'
   replace `name1'_se_`www'_PP=_se[price100_`x'_`ii'_`jj'] in `y'/`y'
  }
  qui replace `name1'_r2a_`www'=e(r2_a) in `y'/`y'
  replace samplesize=e(N) in `y'/`y'
  replace iii=`ii' in `y'/`y'
  replace jjj=`jj' in `y'/`y'
  replace xxx=`x' in `y'/`y'
  replace kkk=`kk' in `y'/`y'
  replace ppp=`pp' in `y'/`y'
  replace www=`ww' in `y'/`y'
  reg `vv'_`x'_`ii'_`jj'`www2'  i.divd  `mkpvar' `ppvar' if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar] 
  replace disp_`name3'_`www'_2=r(sd) in `z'/`z'
  drop e_e
  qui replace divOnly_r2a`name2'`www'=e(r2_a) in `y'/`y'

}
}
}
}

}
}
}
}

local x=0
local jj=2

forvalues zz=1/1000  {
   qui count if  todrop`x'_1_`jj'==0 & fiD==1  & matchprice`x'==1 
   gen zzz_`zz'_1=ceil(runiform()*r(N))
   qui count if  todrop`x'_2_`jj'==0 & fiD==1  & matchprice`x'==1 
   gen zzz_`zz'_2=ceil(runiform()*r(N))
}


/* Now, for the sake of comparison, compute dispersions with sythetically
 generated (i.e., "fake") RHS variables.  This is to construct Table 41 of the paper.  */
qui {
foreach vv in priceI_CFS ltfpqd {

if "`vv'"=="ltfpqd" {
  local name1="tfp"
  local name2="_"
  local name3="TFP"
}
if "`vv'"=="ltfpq3" {
  local name1="phi"
  local name2="_"
  local name3="Phi"
}

if "`vv'"=="priceI_CFS" {
  local name1="price"
  local name2="_P_"
  local name3="Price"
}

forvalues ww=0/1 {   /* do you weight observations by revenue of the downstream plant? */

if `ww'==0 {
  replace weightvar=ones
  local www="UW"
  local www2=""
}
if `ww'==1 {
  replace weightvar=tvsd
  local www="W"
  local www2="_W"
}
forvalues pp=0/2 {    /* the local macro variable pp controls whether we include the local price variable 
                       The default (pp=0) is to not to. */

forvalues ii=1/2 {    /* the local macro variable ii controls whether we discard observations for which there are 
                         many candidate establishments in the downstream zip-code.  The default is not to discard any 
                        observations */

  gen counter=_n
  forvalues kk=0/1 {   /* when k=0 do not include the "price discrimination term */

  local mkpvar = "ones"

  forvalues ccc=1/1000 {
  sort  `vv'_`x'_`ii'_`jj'`www2'
  gen zzz=zzz_`ccc'_`ii'

  gen x2Fake=ones
  gen x1Fake=ones
  if `kk'==1 {
     replace x2Fake=avgMarkupMean[zzz] 
  } 
  if `pp'==1 & `ww'==0 {
    replace x1Fake = price50_`x'_`ii'_`jj'[zzz]
  } 
  if `pp'==2 & `ww'==0 {
     replace x1Fake = price100_`x'_`ii'_`jj'[zzz]
  } 
  if `pp'==1 & `ww'==1 {
     replace x1Fake = price50_`x'_`ii'_`jj'_W[zzz]
  } 
  if `pp'==2 & `ww'==1 {
     replace x1Fake = price100_`x'_`ii'_`jj'_W[zzz]
  } 
  gen tfpqUpFake=tfpq_Up_W_`x'_`ii'_`jj'`www2'[zzz]
  gen x3Fake= divd[zzz]
  qui reg `vv'_`x'_`ii'_`jj'`www2'   x1Fake x2Fake   if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1  [aw=weightvar]
  sort counter
  qui replace disp_FAKE_`name3'_`www'_1=r(sd) in `ccc'/`ccc'
  drop e_e
  qui reg `vv'_`x'_`ii'_`jj'`www2' tfpqUpFake   x1Fake x2Fake   if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  sort counter
  replace disp_FAKE_`name3'_`www'_3=r(sd) in `ccc'/`ccc'
  drop e_e 
  reg `vv'_`x'_`ii'_`jj'`www2' tfpqUpFake    i.x3Fake  x1Fake x2Fake  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar]
  sort counter
  replace disp_FAKE_`name3'_`www'_4=r(sd) in `ccc'/`ccc'
  drop e_e 
  reg `vv'_`x'_`ii'_`jj'`www2'  i.x3Fake  x1Fake x2Fake  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar], robust
  predict e_e, resid
  qui sum e_e  if todrop`x'_`ii'_`jj'==0 & fiD==1  & matchprice`x'==1 [aw=weightvar] 
  sort counter
  replace disp_FAKE_`name3'_`www'_2=r(sd) in `ccc'/`ccc'
  drop e_e zzz  x3Fake x1Fake x2Fake tfpqUpFake
}
 
 foreach var of varlist disp_FAKE_`name3'_`www'_* {
   sort `var'
   replace mn_`var'=1/2 * (`var'[25]+`var'[26]) if iii==`ii' & kkk==`kk' & ppp==`pp' & jjj==2 & xxx==0 
   replace sd_`var'=1/2 * (`var'[975]+ `var'[976] ) if iii==`ii' & kkk==`kk' & ppp==`pp'  & jjj==2 & xxx==0
   replace `var'=.   
}
}
drop counter
}
}
}
}
}

qui drop disp_FAKE*
